@@ -1,14 +1,13 @@
-# -*- encoding : utf-8 -*-
 require './test/test_helper'
 require 'benchmark'
 
 describe 'Performance Poller' do
   X = 10000
   before do
+    REDIS.with { |c| c.respond_to?(:redis) ? c.redis.flushdb : c.flushdb }
     Sidekiq.redis = REDIS
-    Redis.current.flushdb
 
-    #clear all previous saved data from redis
+    # Clear all previous saved data from Redis.
     Sidekiq.redis do |conn|
       conn.keys("cron_job*").each do |key|
         conn.del(key)
@@ -31,7 +30,7 @@ describe 'Performance Poller' do
     Time.stubs(:now).returns(enqueued_time)
   end
 
-  it 'should enqueue 10000 jobs in less than 40s' do
+  it 'should enqueue 10000 jobs in less than 50s' do
     Sidekiq.redis do |conn|
       assert_equal 0, conn.llen("queue:default"), 'Queue should be empty'
     end
@@ -45,6 +44,6 @@ describe 'Performance Poller' do
     end
 
     puts "Performance test finished in #{bench.real}"
-    assert_operator bench.real, :<, 40
+    assert_operator bench.real, :<, 50
   end
 end
