@@ -1,37 +1,39 @@
 require './test/test_helper'
 
-def app
-  Sidekiq::Web
-end
-
 describe 'Cron web' do
   include Rack::Test::Methods
 
+  TOKEN = SecureRandom.base64(32).freeze
+
+  def app
+    Sidekiq::Web
+  end
+
   before do
+    env 'rack.session', { csrf: TOKEN }
+    env 'HTTP_X_CSRF_TOKEN', TOKEN
+    REDIS.with { |c| c.respond_to?(:redis) ? c.redis.flushdb : c.flushdb }
     Sidekiq.redis = REDIS
-    Redis.current.flushdb
+  end
 
-    #clear all previous saved data from redis
-    Sidekiq.redis do |conn|
-      conn.keys("cron_job*").each do |key|
-        conn.del(key)
-      end
-    end
+  let(:job_name) { "TestNameOfCronJob" }
+  let(:cron_job_name) { "TesQueueNameOfCronJob" }
 
-    @args = {
-      name: "TestNameOfCronJob",
+  let(:args) do
+    {
+      name: job_name,
       cron: "*/2 * * * *",
       klass: "CronTestClass"
     }
+  end
 
-
-    @cron_args = {
-      name: "TesQueueNameOfCronJob",
+  let(:cron_args) do
+    {
+      name: cron_job_name,
       cron: "*/2 * * * *",
       klass: "CronQueueTestClass",
       queue: "cron"
     }
-
   end
 
   it 'display cron web' do
@@ -45,7 +47,7 @@ describe 'Cron web' do
   end
 
   it 'display cron web with cron jobs table' do
-    Sidekiq::Cron::Job.create(@args)
+    Sidekiq::Cron::Job.create(args)
 
     get '/cron'
     assert_equal 200, last_response.status
@@ -55,27 +57,25 @@ describe 'Cron web' do
   end
 
   describe "work with cron job" do
-
     before do
-      @job = Sidekiq::Cron::Job.new(@args.merge(status: "enabled"))
-      @job.save
-      @name = "TestNameOfCronJob"
+      @job = Sidekiq::Cron::Job.new(args.merge(status: "enabled"))
+      assert @job.save
 
-      @cron_job = Sidekiq::Cron::Job.new(@cron_args.merge(status: "enabled"))
-      @cron_job.save
-      @cron_job_name = "TesQueueNameOfCronJob"
+      @cron_job = Sidekiq::Cron::Job.new(cron_args.merge(status: "enabled"))
+      assert @cron_job.save
     end
 
     it 'shows history of a cron job' do
       @job.enque!
-      get "/cron/#{@name}"
+      get "/cron/#{job_name}"
 
       jid =
         Sidekiq.redis do |conn|
-          history = conn.lrange Sidekiq::Cron::Job.jid_history_key(@name), 0, -1
+          history = conn.lrange Sidekiq::Cron::Job.jid_history_key(job_name), 0, -1
           Sidekiq.load_json(history.last)['jid']
         end
 
+      assert jid
       assert last_response.body.include?(jid)
     end
 
@@ -87,18 +87,18 @@ describe 'Cron web' do
 
     it "disable and enable all cron jobs" do
       post "/cron/__all__/disable"
-      assert_equal Sidekiq::Cron::Job.find(@name).status, "disabled"
+      assert_equal Sidekiq::Cron::Job.find(job_name).status, "disabled"
 
       post "/cron/__all__/enable"
-      assert_equal Sidekiq::Cron::Job.find(@name).status, "enabled"
+      assert_equal Sidekiq::Cron::Job.find(job_name).status, "enabled"
     end
 
     it "disable and enable cron job" do
-      post "/cron/#{@name}/disable"
-      assert_equal Sidekiq::Cron::Job.find(@name).status, "disabled"
+      post "/cron/#{job_name}/disable"
+      assert_equal Sidekiq::Cron::Job.find(job_name).status, "disabled"
 
-      post "/cron/#{@name}/enable"
-      assert_equal Sidekiq::Cron::Job.find(@name).status, "enabled"
+      post "/cron/#{job_name}/enable"
+      assert_equal Sidekiq::Cron::Job.find(job_name).status, "enabled"
     end
 
     it "enqueue all jobs" do
@@ -119,21 +119,21 @@ describe 'Cron web' do
         assert_equal 0, conn.llen("queue:default"), "Queue should have no jobs"
       end
 
-      post "/cron/#{@name}/enque"
+      post "/cron/#{job_name}/enque"
 
       Sidekiq.redis do |conn|
         assert_equal 1, conn.llen("queue:default"), "Queue should have 1 job"
       end
 
-      #should enqueue more times
-      post "/cron/#{@name}/enque"
+      # Should enqueue more times.
+      post "/cron/#{job_name}/enque"
 
       Sidekiq.redis do |conn|
         assert_equal 2, conn.llen("queue:default"), "Queue should have 2 job"
       end
 
-      #should enqueue to cron job queue
-      post "/cron/#{@cron_job_name}/enque"
+      # Should enqueue to cron job queue.
+      post "/cron/#{cron_job_name}/enque"
 
       Sidekiq.redis do |conn|
         assert_equal 1, conn.llen("queue:cron"), "Queue should have 1 cron job"
@@ -142,8 +142,8 @@ describe 'Cron web' do
 
     it "destroy job" do
       assert_equal Sidekiq::Cron::Job.all.size, 2, "Should have 2 job"
-      post "/cron/#{@name}/delete"
-      post "/cron/#{@cron_job_name}/delete"
+      post "/cron/#{job_name}/delete"
+      post "/cron/#{cron_job_name}/delete"
       assert_equal Sidekiq::Cron::Job.all.size, 0, "Should have zero jobs"
     end
 
